*-///////////////////////////////////////////////////////////////////////////////////
*- 
*- Create a PRG from a VCX Class Library
*-
*- Author - Eugene Vital. 
*- 
*-
*- Version .1.1b --- 10/03/2007
*-    Added code to generate method code for member objects of a class
*-
*- Version .1.0b --- 12/09/2005
*-    Original release
*-
*-
*- Example
*- SET POCEDURE TO X2PRG.prg
*- loX2PRG = CREATEOBJECT('X2PRG')
*- loX2PRG.Convert("someclasslibrary.vcx", 'somenewprg.prg')
*- or
*- Optionally specify a single class to convert
*- loX2PRG.Convert("someclasslibrary.vcx", 'somenewprg.prg', "justhisclass")
*-
*-
*- Beta version so use with caution. ;) 
*-
*- This code creates a working copy of the original VCX to reduce the risk of corruption
*- but I recommend you back up yout VCX first.
*-
*-
*-
*- Warranty Disclaimer: NO WARRANTY!!!
*////////////////////////////////////////////////////////////////////////////////////


DEFINE CLASS X2PRG as Session OLEPUBLIC 

	PROCEDURE Init()
		SET DELETED ON
		SET EXACT OFF
	ENDPROC 

	*------------------------------------------------------------------------
	*-
	*- Procedure Convert() 
	*- 	Parameters :	tcVCX - The name of the VCX classlib to work with
	*-								tcPRG - The name of the output PRG file
	*-								tcClass - Optional parameter to specify a single class to export
	*-
	*------------------------------------------------------------------------
 	PROCEDURE Convert(tcVCX as String , tcPRG as String , tcClass as String )

		*/Local vars for this methd
		LOCAL lcReturnValue as string ,;
					lcVCX as String ,;
					lcPRG as String ,;
					lcClass as String ,;
					lcPRGText as String,;
					liCount as Integer ,;
					xx as Integer,;
					yy as Integer 
					
		LOCAL ARRAY laProperties[1]			
		LOCAL ARRAY laProtected[1]
		LOCAL ARRAY laMethods[1]
		
		LOCAL ARRAY laMembers[1]
		LOCAL ARRAY laProtectedMembers[1]

				
		lcReturnValue = .t.
		
				
		*/
		*/Parameter validation
		*/
		IF VARTYPE(tcVCX) <>"C" OR !FILE(tcVCX)		
			lcReturnValue = .f.
			RAISEEVENT(this, "MessageEvent", "Invalid ClassLib specified")
		ELSE
			lcVCX = tcVCX	
		ENDIF 

		IF VARTYPE(tcPRG) <>"C"
			lcReturnValue = .f.
			RAISEEVENT(this, "MessageEvent", "Invalid ClassLib specified")
		ELSE
			lcPRG = tcPRG	
		ENDIF 

		IF VARTYPE(tcClass) <>"C"
			lcClass = ""
		ELSE
			* class names are stored in lowercase
			lcClass = LOWER(tcClass)
		ENDIF 

		
		*/
		*/If parameters passed validation it is ok to continue
		*/
		IF lcReturnValue  


		
			*/
			*/ create the working cursor, if a spcecific class name was specified just export that one,
			*/ otherwise select all the records
			TRY 
				IF lcClass == ""
					SELECT *;
						FROM (lcVCX);
						WHERE reserved1 == "Class"  AND DELETED() =.f.;
						INTO CURSOR tmpVCX 
				ELSE
					SELECT * ;
						FROM (lcVCX);
						WHERE reserved1 == "Class"  AND objname == lcClass  AND DELETED() =.f.;
						INTO CURSOR tmpVCX 
				ENDIF 
				CATCH 
					lcReturnValue = .f.
					RAISEEVENT(this, "MessageEvent", "Could not open the file " + lcVCX)			
				ENDTRY 				


			*/
			*/ A simple validation that we are working with a class library
			*/
			IF lcReturnValue = .t.
				RAISEEVENT(this, "MessageEvent", "File " +  lcVCX + " IS a valid Visual FoxPro class library.")

				*/
				*/ Scan the temp cusrsor of valid class definition records
				*/	
				
				SET TEXTMERGE TO MEMVAR lcPRGText NOSHOW 
				SET TEXTMERGE ON
				SCAN 
					
					*/
					*/ the reserved8 field is used for the include file
					*/
					IF !EMPTY(reserved8)
						*\<<"*/include files for this class">>
						*\<<"#include " + tmpVCX.reserved8 >> 
					ENDIF 
					 	
				
					*RAISEEVENT(this, "MessageEvent", "Converting class " + 	tmpVCX.objname)
					
					
					*/
					*/ Beginning class definition
					*/
					\<<"DEFINE CLASS " + tmpVCX.objname  + " AS " + tmpVCX.class + IIF(!EMPTY(tmpVCX.classloc) , " OF "+ tmpVCX.classloc , "")>>
					
					
					*//////////////////////////////////////////////////////
					*/ The following extracts the Properties for the class
					*/
					
						*/
						*/ Array is used for determining if properties/methods are hidden/protected
						*/
						ALINES(laProtected, tmpVCX.protected, 5)
						liCount = ALINES(laProperties, tmpVCX.properties, 5)

						*/
						*/ Check for hidden protected attributes
						*/					
						FOR xx = 1 to liCount
							yy = ASCAN(laProtected, ALLTRIM(GETWORDNUM(laProperties[xx], 1, "=")))
							IF yy > 0 
								*- if it is, prefix the property name. A Carret (^) sufix signifies a HIDDEN property
									\  <<IIF(RIGHT(laProtected[yy],1) ="^", "HIDDEN", "PROTECTED")>> <<ALLTRIM(GETWORDNUM(laProperties[xx], 1, "="))>>
							ENDIF
						NEXT xx

						*/
						*/ Assign property values
						*/
						liCount = ALINES(laProperties, tmpVCX.properties, 5)
						FOR xx = 1 to liCount
							*- output the property name and value
							\  <<this.FixProperty(laProperties[xx])>>
						NEXT xx
					


					*//////////////////////////////////////////////////////
					*/ Extract the member objects for this class
					*/
					SELECT * ;
						FROM (lcVCX);
						WHERE parent == tmpVCX.objname AND DELETED() =.f. ;
						INTO CURSOR tmpVCXMembers

					SELECT tmpVCXMembers
					
					ALINES(laProtectedMembers, tmpVCX.reserved3, 5)
					

					SCAN
                        \  <<"*\">>
                        \  <<"*\ Member Object " + tmpVCXMembers.objname>>
                        \  <<"*\">>
						\  <<"ADD OBJECT " +IIF(ASCAN(laProtectedMembers, tmpVCXMembers.objname)>0, "PROTECTED ", " ")  + tmpVCXMembers.objname +' AS ' + tmpVCXMembers.class )>>
						*/						*/
						*/ Assign property values for the member objects
						*/
						liCount = ALINES(laProperties, tmpVCXMembers.properties, 5)
						IF liCount > 0
							\\ <<"WITH;">>
						ENDIF 
						FOR xx = 1 to liCount
							*- output the property name and value
							\    <<this.FixProperty(laProperties[xx])>>
							IF xx < liCount
							\\<<",;">>
							ENDIF 
						NEXT xx
                        \
                        */
                        */ Output Methods for the member object
                        */
                        liCount = ALINES(laMethods, tmpVCXMembers.methods, 5)
                        FOR xx = 1 to liCount
                            DO CASE
                              CASE LEFT(ALLTRIM(laMethods[xx]), 10) == "PROCEDURE " 
                                \    <<STRTRAN(laMethods[xx], "PROCEDURE ", "PROCEDURE " + tmpVCXMembers.objname + ".") + "()">>
                              CASE LEFT(ALLTRIM(laMethods[xx]), 7) == "ENDPROC"
                                \    <<laMethods[xx]>>
                                \
                              OTHERWISE
                                \        <<laMethods[xx]>>
                            ENDCASE
                        NEXT xx

                        
                        *\<<STRTRAN(tmpVCXMembers.methods, "PROCEDURE ", "PROCEDURE " + tmpVCXMembers.objname + ".") >>
                        
						RAISEEVENT(this, "MessageEvent",  tmpVCXMembers.objname)
					ENDSCAN 


					*//////////////////////////////////////////////////////
					*/ Extract the Methods for the class
					*/
                    *- make sure we set focus back to the correct cursor
                    SELECT tmpVCX
                    
					IF !EMPTY(ALLTRIM(tmpVCX.methods))
	  				liCount = ALINES(laMethods, tmpVCX.methods, 5)
						FOR xx = 1 to liCount
                            DO CASE
                              CASE LEFT(ALLTRIM(laMethods[xx]), 10) == "PROCEDURE " 
                                *- check to see if the method name  is in the protected array
                                yy = ASCAN(laProtected, ALLTRIM(GETWORDNUM(laMethods[xx], 2, " "))) 
                                IF yy > 0 
                                  *- if it is, prefix the method name. A Carret (^) sufix signifies a HIDDEN method
                                  laMethods[xx] = IIF(RIGHT(laProtected[yy],1) ="^", "HIDDEN ", "PROTECTED ") + laMethods[xx] 
                                ENDIF
                                \  <<laMethods[xx]+"()">>
                              CASE LEFT(ALLTRIM(laMethods[xx]), 7) == "ENDPROC"
                                \  <<laMethods[xx])>>
                                \
                              
                              OTHERWISE
                                \    <<laMethods[xx])>>
                            ENDCASE
                            
                            
						NEXT xx
					ENDIF 	
					*/
					*/ End of class definition
					*/
					\<<"ENDDEFINE &&" + tmpVCX.objname  >>
					\
					\
				ENDSCAN
				
				
	
				SET TEXTMERGE off
				SET TEXTMERGE TO 
								
	
				STRTOFILE(lcPRGText, lcPRG, 0 )
				


				lcReturnValue = .t.
			ELSE
				lcReturnValue = .f.
				 RAISEEVENT(this, "MessageEvent", "File " +  lcVCX + " is NOT a valid Visual FoxPro class library.")
			ENDIF 
			
			
		ENDIF 
		RETURN lcReturnValue 
	ENDPROC 
	
	
	
	*------------------------------------------------------------------------
	*-
	*- Properties that require special handling. Some don't get quotes etc...
	*- This code is fuzzy making some assumptions
	*-
	*------------------------------------------------------------------------
	PROTECTED PROCEDURE FixProperty( tcPropertyString )	
		LOCAL lReturnValue as String ,;
					lcPropertyName as String ,;
					lvPropertyValue as Variant 	
					
		
		* parse the passed in values and extract the name and value
		lcPropertyName = ALLTRIM(GETWORDNUM( tcPropertyString, 1, "=")) 
		lvPropertyValue = ALLTRIM(GETWORDNUM( tcPropertyString, 2, "=")) 
		
		lReturnValue = tcPropertyString 
		
		
		 
		DO CASE 

			CASE OCCURS("color",  LOWER( lcPropertyName )) >0 AND GETWORDCOUNT( lvPropertyValue, ",") = 3
			* some kind of color property so we need to put the function definition around the the value
				lReturnValue = lcPropertyName + " = RGB(" + lvPropertyValue +")"
			
			
			CASE ISDIGIT(lvPropertyValue) 
				* a number of some kind	so there isn't anything to do 
				lReturnValue = tcPropertyString


			CASE LOWER(lvPropertyValue) == ".t." OR LOWER(lvPropertyValue) == ".f."  
				* a logical value so there isn't anything to do 
				lReturnValue = tcPropertyString


			CASE LEFT(lvPropertyValue,1)="(" AND RIGHT(lvPropertyValue,1) = ")"
				*when an property is set using the = operator it is saved in enclosed paranthasese so we can leave it alone 
				lReturnValue = tcPropertyString

			OTHERWISE 
			* string values need to be quoted if they aren't allready
			IF OCCURS('"', lvPropertyValue)=0
				lReturnValue = lcPropertyName  + '= "' + lvPropertyValue + '"'
			ENDIF 		
			
		ENDCASE 

		RETURN lReturnValue
	ENDPROC 

	
	*------------------------------------------------------------------------
	*-
	*- Event Procedure used for passing messages.
	*-
	*------------------------------------------------------------------------
	PROCEDURE MessageEvent(tcMessageString as String )
		*- Example of using the BINDEVENTS() function to use this feature.
		*- BINDEVENT(oSource, "MessageEvent", oHandler, "MessageEvent")
	ENDPROC 
ENDDEFINE 



* VCX fields and their properties

* reserved1 - 
* reserved2 - 
* reserved3 - 
* reserved4 - 
* reserved5 - 
* reserved6 - 
* reserved7 - 
* reserved8 - include file